package aima.cs.machine;

import java.io.*;
import aima.stat.rv.*;

/**
  * A probablistic FSM with integer input/output.
  *
  * @author Daishi Harada (daishi@cs.berkeley.edu)
  */
public class intPFSM
  implements intMachine
{
  public intPFSM() {
  }

  public intPFSM(int numStates, int numInputs, int numOutputs) {
    this.numStates = numStates;
    this.numInputs = numInputs;
    this.numOutputs = numOutputs;
    transition = new BiasedNDie[numStates][numInputs];
    output = new BiasedNDie[numStates];
  }

  public intPFSM(int numStates, int numInputs, int numOutputs,
		 double[][][] transitionProbability, double[][] outputProbability) {
    this(numStates,numInputs,numOutputs);
    int i,j,k;
    for (i=0;i<numStates;i++) {
      for (j=0;j<numInputs;j++) {
	transition[i][j] = new BiasedNDie(numStates,transitionProbability[i][j]);
	}
      output[i] = new BiasedNDie(numOutputs,outputProbability[i]);
    }
  }

  public int step(int input) {
    state = transition[state][input].sample();
    return (output[state].sample());
  }

  public Object stepObject(Object input) {
    return (new Integer(step(((Integer)input).intValue())));
  }

  public void setState(int state) {
    this.state = state;
  }

  public int getState() {
    return (state);
  }

  private int state;
  private int numStates;
  private int numInputs;
  private int numOutputs;
  private BiasedNDie[][] transition;
  private BiasedNDie[] output;

  private int verbosity = 0;
  private PrintStream logStream = System.err;
}
