package aima.examples;

import aima.search.*;
import java.lang.*;
import java.util.*;
import aima.cs.util.*;

public class TestSearch {

    public static void main(String argv[]) {
      
       System.out.println("Trivial search space based on adding 2 or 3");

       System.out.println("DFS:");

       listPath( ( new DepthFirstSearch(new TwoThreeState()) ).search() );
     
       System.out.println();
       System.out.println("Depth Bounded (depth 7):");

       listPath( ( new DepthBoundedSearch(new TwoThreeState(),7) ).search() );

       System.out.println();
       System.out.println("BFS:");

       listPath( ( new BreadthFirstSearch(new TwoThreeState()) ).search() );
       System.out.println();

       System.out.println();
       System.out.println("Uniform cost:");

       listPath( ( new UniformCostSearch(new TwoThreeState()) ).search() );

       System.out.println();
       System.out.println("Iterated Deepening Search:");

       listPath( ( 
           new IteratedDeepeningSearch(new TwoThreeState()) ).search() );

       System.out.println();
       System.out.println("A* Search:");

       listPath( ( 
           new AStarSearch(new TwoThreeState()) ).search() );

       System.out.println();
       System.out.println("Greedy Search:");

       listPath( ( 
           new GreedySearch(new TwoThreeState()) ).search() );

       System.out.println();

       System.out.println("Generalized missionaries and cannibals (3,3,1)");

       System.out.println("Breadth-first search skipping repeated states");
       listPath( ( 
           new GeneralQueueSearch(new SearchNode( new MCState(3,3,1) ), 
                                  new NoRepeatQueue( new FIFOQueue() ) 
                                   ) ).search() );
       System.out.println();

       System.out.println("Breadth-first search with repeated states");
       listPath( ( new BreadthFirstSearch(new MCState(3,3,1)) ).search() );
       System.out.println();

       System.out.println("A* Search:");
       listPath( ( new AStarSearch(new MCState(3,3,1)) ).search() );
       System.out.println();

       System.out.println("Greedy Search:");
       listPath( ( new GreedySearch(new MCState(3,3,1)) ).search() );
       System.out.println();
    }


    protected static void listPath(SearchNode node) {
         
        if (node == null) {
            System.out.println("No solution");
            return;
        }

        while (node.getParent()!=null) {

           System.out.println( "State: " + node.getState() +
                              " Depth: " + node.getDepth() +
                              " Cost: " + node.getPathCost() +
                              " by applying " + node.getAppliedOp()
                              ); 

           node = node.getParent();

        }

        
        System.out.println( "Starting at state: " + node.getState());

    }  

}
