package aima.stat.rv;

/**
  * A biased n-sided die.
  *
  * This is the most trivial subclass of the class <code>FiniteSampler</code>.
  * It essentially adds no new functionality; it wraps the <code>FiniteSampler</code>
  * with the <code>intRV</code> interface.
  *
  * @see FiniteSampler
  * @author Daishi Harada (daishi@cs.berkeley.edu)
  */
public class BiasedNDie
  extends FiniteSampler
  implements intRV
{
  /**
    * Constructor initializing only the random source.
    */
  public BiasedNDie() {
    super();
  }

  /**
    * Constructor initializing the random source and the number of sides on the die.
    */
  public BiasedNDie(int n) {
    super(n);
  }

  /**
    * Constructor initializing the random source, the number of sides, and the distribution.
    */
  public BiasedNDie(int n, double[] p) {
    super(n,p);
  }

  /**
    * Sample/roll the die.
    *
    * @return i with probability prob[i] (0 <= i < n.)
    */
  public int sample() {
    return (indexSample());
  }

  /**
    * Sample/roll the die.
    *
    * @return Integer(i) with probability prob[i] (0 <= i < n.)
    */
  public Object sampleObject() {
    return (new Integer(sample()));
  }
}
