package aima.stat.rv;

import java.util.*;

/**
  * An n-sided die.
  *
  * Sampling returns one of {0,1,...,n-1} with (approximately) equal probability.
  *
  * @author Daishi Harada (daishi@cs.berkeley.edu)
  */
public class NDie
  implements intRV
{
  /**
   * Constructor initializing only the random source.
   */
  public NDie() {
    randomSource = new Random();
  }

  /**
    * Constructor initializing the random source and the number of sides on the die.
    */
  public NDie(int n) {
    this();
    this.n = n;
  }

  /**
    * Sample/roll the die.
    *
    * @return i with probability (approximately) 1/n.
    */
  public int sample() {
    return ((int)(n*randomSource.nextDouble()));
  }

  /**
    * Sample/roll the die.
    *
    * @return Integer(i) with probability (approximately) 1/n.
    */
  public Object sampleObject() {
    return (new Integer(sample()));
  }

  public void setN(int n) {
    this.n = n;
  }

  public int getN() {
    return (n);
  }

  private int n;
  private Random randomSource;
}
