package aima;

/**
  * The Agent interface.
  *
  * Note that both the Agent and the Agent.Program
  * have identically signatured execute methods.
  * The Agent's execute method is expected to call
  * the Agent.Program's execute method. In addition,
  * however, this allows us to conceptually partition
  * between "algorithmic" and "cosmetic" aspects of execution.
  * The agent program is responsible for all the
  * algorithmic aspects. If there are any other things
  * that one wants an agent to do
  * (e.g., change the graphic representing the Agent
  * when agent is doing action "foo")
  * then that should be done at the Agent's execute method.
  * Such attributes are supposed to be part of the Agent.Body.
  */
public interface Agent
{
  /**
    * The Agent's execute method.
    */
  public abstract Action execute(Percept percept);
  
  /**
    * The interface specifying the Agent's Program.
    */
  public interface Program {
    /**
      * Compute agent's response to percept (if any).
      */
    public abstract Action execute(Percept percept);
  }

  /**
    * The interface naming Agent Bodies.
    * This is encapsulates the "physical"
    * manifestation of the agent inside the environment.
    * It is provided to the agent by the environment.
    * It contains the Actions and Percepts available
    * to the agent.
    */
  public interface Body
    extends Environment.Thing
  {
    public Set getActions();
    public Set getPercepts();
  }
}
