import aima.search.*;
import java.lang.*;
import java.util.*;

public class MCState implements State, Heuristic {

   int missionariesLeft, cannibalsLeft, boatsLeft;
   int missionariesOver, cannibalsOver, boatsOver;

   public MCState(int missionariesLeft, int cannibalsLeft, int boatsLeft,
                  int missionariesOver, int cannibalsOver, int boatsOver) {

       this.missionariesLeft = missionariesLeft;
       this.cannibalsLeft = cannibalsLeft;
       this.boatsLeft = boatsLeft;

       this.missionariesOver = missionariesOver;
       this.cannibalsOver = cannibalsOver;
       this.boatsOver = boatsOver;

   } 

   public MCState(int missionariesLeft, int cannibalsLeft, int boatsLeft) {
       this(missionariesLeft, cannibalsLeft, boatsLeft, 0, 0, 0);
   }


   protected boolean isValid() {

         if (boatsLeft < 0 || boatsOver < 0 ||
             missionariesLeft < 0 || missionariesOver < 0 ||
             cannibalsLeft < 0 || cannibalsOver < 0) return false;

         if (missionariesLeft > 0 
                 && cannibalsLeft > missionariesLeft) return false;

         if (missionariesOver > 0 
                 && cannibalsOver > missionariesOver) return false;

         return true;

   } 

   public boolean isGoal() {

      return  missionariesLeft == 0 &&
               cannibalsLeft == 0 &&
               boatsLeft == 0 ;

   }


  public Enumeration successors() { 

    int dMissionaries, dCannibals, dBoats;

    Vector successorVec = new Vector();

    for (dBoats= -1; dBoats<=1; dBoats+=2) {
      for (dMissionaries=0; dMissionaries<=2; dMissionaries++) {
	for (dCannibals=0; dCannibals+dMissionaries<=2; dCannibals++) {
	  if ( dMissionaries+dCannibals>0 ) {

             MCState tryState = 
                    new MCState(missionariesLeft+dBoats*dMissionaries,
                                cannibalsLeft+dBoats*dCannibals,
                                boatsLeft+dBoats,
                                missionariesOver-dBoats*dMissionaries,
                                cannibalsOver-dBoats*dCannibals,
                                boatsOver-dBoats);
                                
	    if (tryState.isValid()) {

	      successorVec.addElement(
			  new Successor(
                          tryState, 
			  "(" + dMissionaries + "," 
                              + dCannibals + "," 
                              + dBoats + ")", 1 )
			 ); 
	    }
	  }  
	}
      }
    }

    return successorVec.elements();
  }

  public float h() {

     float hVal = (float) missionariesLeft + (float) cannibalsLeft
                  - (float) boatsLeft;

     return hVal;
            
  }

  public String toString() {

      return "(" + missionariesLeft + ":" + missionariesOver + "," +
                 + cannibalsLeft + ":" + cannibalsOver + "," +
                 + boatsLeft + ":" + boatsOver + ")";

  }

  

}
