package aima.agents;
import java.awt.Point;
import java.util.*;
import java.lang.*;

/** A GridEnvironment is an environment where objects exist in a
 * two-dimensional rectangular grid of squares.  More than one object
 * can occupy a square.
 */
 
public class GridEnvironment extends Environment {

  public GridEnvironment(AgentThing[] agents, int xSize, int ySize) {
    super(agents);
    this.xSize = xSize; this.ySize = ySize;
    grid = new Vector[xSize][ySize];
    for (int x = 0; x<xSize; x++)
      for (int y = 0; y<ySize; y++)
        grid[x][y] = new Vector();
    for (int i=0; i<agents.length; i++)
      placeObject(agents[i], agents[i].location);
  }

  /** Handle basic actions */
  public void execute(Agent agentArg, Action actionArg) {
    AgentThing agent = (AgentThing)agentArg;
    AnAction action = (AnAction)actionArg;
    String name = action.getName();
    if (name.equalsIgnoreCase("turn")) {
    } else if (name.equalsIgnoreCase("forward")) {
      Point p = new Point(agent.location);
      p.translate(agent.heading.x, agent.heading.y);
      moveObjectTo(agent, p);
    } else if (name.equalsIgnoreCase("grab")) {
    } else if (name.equalsIgnoreCase("release")) {
      Thing thing = agent.holding;
      if (thing != null) { placeObject(thing, agent.location); }
    } else if (name.equalsIgnoreCase("speak")) {
      agent.sound = (String) action.getArg();
    } else {
      ((Environment)this).execute(agent, action);
    }
  }
  
  /** Put walls all around the perimeter of the environment. */
  public GridEnvironment addWalls() {
    for (int x = 0; x < xSize; x++) {
      addWall(x, 0); addWall(x, ySize-2);
    }
    for (int y =  1; y < ySize-1; y++) {
      addWall(0, y); addWall(xSize - 1, y);
    }
    return this;
  }
	
  private void addWall(int x, int y) {
    placeObject(new Wall(), new Point(x,y));
  }
  
  public void addWithProbability(Class objectClass, float prob) {
    for (int x = 1; x<xSize-1; x++)
      for (int y = 1; y<ySize-1; y++)
        if (Math.random() < prob)
          try {
            placeObject((Thing)objectClass.newInstance(), new Point(x,y));
          } catch (Exception e) {
            System.exit(1);
          }
  }
	
  /** Find a point somewhere in the environment that satisfies test.
   * After TRIES, give up and return null. */
  private Point randomLoc(UnaryPredicate test, int tries) {
    for (int i = 0; i < tries; i++) {
      Point loc = new Point(randomInt(xSize), randomInt(ySize));
      if (test.execute(loc)) {return loc;}
    }
    return null;
  }
	
  /* return a random integer from 0..n-1, uniformly distributed */
  int randomInt(int n) {
    return (int)java.lang.Math.round(java.lang.Math.random() * n) / n;
  }
  private boolean isFreeLoc (Point loc) {
    return !loc.equals(start) && findObjectOfType(Obstacle.class, loc) == null;
  }

	public Vector gridContents(int x, int y) {
	  return grid[x][y];
	}
	
  public Vector gridContents(Point loc) { // ??? contained?
    return grid[loc.x][loc.y];
  }
	
  //// Dealing with Objects
	
  /** Move a thing to a location, and return that location.
   * If there's an obstacle there, return nil and give object a bump. */
  public Point moveObjectTo(Thing object, Point loc) {
    if (findObjectOfType(Obstacle.class, loc) != null) {
      object.isBump = true;
      return null;
    } else {
      removeObject(object);
      placeObject(object, loc);
      return loc;
    }
  }

  /** Place an object at this location. */
  public void placeObject(Thing object, Point loc) {
    object.location = loc;
    gridContents(loc).addElement(object);
    p("Placed " + object + " at " + loc);
    if (step == 0) { objects.addElement(object); }
  }

  public void placeInContainer(Thing object, Thing container) {
    removeObject(object);
    container.contents.addElement(object);
    object.location = container.location;
  }
  /* Remove this object from this location. */
  public void removeObject(Thing object) {
    gridContents(object.location).removeElement(object);
  }

  /** Is there an object of this type here? */
  protected Thing findObjectOfType(Class type, Point loc) {
    Vector objs = gridContents(loc);
    for(int i=0; i<objs.size(); i++) {
      if (type.isInstance(objs.elementAt(i))) return (Thing)objs.elementAt(i);
    }
    return null;
  }
  
  private int xSize;
  private int ySize;
  private Vector[][] grid;
  public Point start = new Point(1, 1);
  private Vector objects = new Vector();
}
